typedef struct DmaBuffer {
	short		*buffer;	/* The data buffer		*/
	short		*bufp;		/* Next data to xfer to user	*/
	short		*fillp;		/* Next data to xfer from ADC	*/
	int		size;		/* Buffer size (in words)	*/
	int		count;		/* Number of words in buffer	*/
	volatile int	full;		/* Is the buffer full?		*/
} DmaBuffer_t;

typedef struct ADC_t {
	/* Driver flags */
	int		open;		/* Is device open?		*/
	int		first_time;	/* Flag for first access	*/
	int		status;		/* Status of ADC driver		*/
	/* IO port addresses for ADC control registers */
	int		fifo;		/* Address of Data FIFO port	*/
	int		clear;		/* Address of ADC clear port	*/
	int		start;		/* Address of Start Convert port*/
	int		dmaintclr;	/* Address of Interrupt clear 	*/
	int		intvlcntr;	/* Address of Interval Counter 	*/
	int		intvlstrobe;	/* Address of Interval Strobe	*/
	/* ADC acquisition configuration parameters */
	int		dmaen;		/* Enable DMA transfers		*/
	int		clock;		/* Clock for counter		*/
	int		intvlclock;	/* Value for invtl clock (B1)	*/
	int		intvlcount;	/* Value for invtl mode		*/
	int		scan;		/* Enable ADC input scanning	*/
	int		gain;		/* ADC input gain		*/
	int		polarity;	/* Bipolar or unipolar input	*/
	int		channel;	/* Input channel number		*/
	int		timebase;	/* Select clock souce for ADC	*/
	int		triggermode;	/* Select ADC trigger mode	*/
	int		autoacquire;	/* Auto acquire on read()	*/
	int		inputmode;	/* Single ended/Diffl input	*/
	int		eckdrv;		/* Mode for EXTCONV line	*/
	int		intscan;	/* Enable interval scanning	*/
	int		waitfortrig;	/* Wait for software trigger	*/
	/* Data acquisition buffers */
	DmaBuffer_t	*buflist;	/* List of dma buffers		*/
	DmaBuffer_t	*head;		/* Buffer in list for DMA xfer	*/
	DmaBuffer_t	*tail;		/* Buffer in list for emptying	*/
	int		bufsize;	/* Size of buffers		*/
	int		nbuf;		/* Number of buffers		*/
	int		residue;	/* Words left to DMA xfer	*/
	int		count;		/* Words left to user xfer	*/
	/* Device driver resources */
	int		owndma;		/* Do we own the dma channel	*/
	struct file	*file;		/* File for acquisition		*/
	struct semaphore sem;		/* Semaphore to lock port access*/
	struct wait_queue *wait;	/* Wait queue for read()	*/
	struct wait_queue *exwait;	/* Wait queue for exceptions	*/
	struct LabPC_t	*board;
} ADC_t;

extern void adc_interrupt( ADC_t *adc, int status );
extern ADC_t *get_adc( LabPC_t *board, int minor );
extern void free_adc( ADC_t *adc );
