/*
 * $Id: labpc.h,v 0.5 1996/11/07 14:09:49 glenn Exp glenn $
 *
 */
 
#ifndef _LINUX_LABPC_H
#define _LINUX_LABPC_H

/* minor device numbers (make sure to get these right when doing mknod :) */
#ifndef DAQ_LABPC
#define DAQ_LABPC		0	/* Card master device		*/
#endif
#ifndef DAQ_ADC
#define DAQ_ADC			1	/* Analogue to Digital Converter*/
#endif
#ifndef DAQ_DAC0
#define DAQ_DAC0		2	/* Digital to Analogue Converter*/
#endif
#ifndef DAQ_DAC1
#define DAQ_DAC1		3	/* Digital to Analogue Converter*/
#endif
#ifndef DAQ_COUNTERA0
#define DAQ_COUNTERA0		4	/* The 8253 counter/timers	*/
#endif
#ifndef DAQ_COUNTERA1
#define DAQ_COUNTERA1		5
#endif
#ifndef DAQ_COUNTERA2
#define DAQ_COUNTERA2		6
#endif
#ifndef DAQ_COUNTERB0
#define DAQ_COUNTERB0		7
#endif
#ifndef DAQ_COUNTERB1
#define DAQ_COUNTERB1		8
#endif
#ifndef DAQ_COUNTERB2
#define DAQ_COUNTERB2		9
#endif
#ifndef DAQ_DPORTA
#define DAQ_DPORTA		10	/* The 8255 digital IO ports	*/
#endif
#ifndef DAQ_DPORTB
#define DAQ_DPORTB		11
#endif
#ifndef DAQ_DPORTC
#define DAQ_DPORTC		12
#endif
#ifndef DAQ_NUMDEVS
#define DAQ_NUMDEVS		13	/* number of devices supported 	*/
#endif

#ifdef __KERNEL__

#include <sys/ioctl.h>

/* The Factory default settings - override these on the command line */
#ifndef LABPC_BASE
#define	LABPC_BASE		0x260
#endif
#ifndef LABPC_IRQ
#define LABPC_IRQ		5
#endif
#ifndef LABPC_DMA
#define LABPC_DMA		3
#endif

/* refer to manual for descriptions of what they do and r/w assignments */
/* Configuration and Status group	*/
#define DAQ_CMD1		0x00
#define DAQ_CMD2		0x01
#define DAQ_CMD3		0x02
#define DAQ_CMD4		0x0f
#define DAQ_STAT		0x00
/* Analogue Input group			*/
#define DAQ_FIFO		0x0a
#define DAQ_ADCCLR		0x08
#define DAQ_STRTCVT		0x03
#define DAQ_DMATCCLR		0x0a
/* Analogue Ouput group			*/
#define DAQ_DAC0LO		0x04
#define DAQ_DAC0HI		0x05
#define DAQ_DAC1LO		0x06
#define DAQ_DAC1HI		0x07
/* Counter/Timer Group A		*/
#define DAQ_CTA0		0x14
#define DAQ_CTA1		0x15
#define DAQ_CTA2		0x16
#define DAQ_CTAMODE		0x17
#define DAQ_TICLR		0x0c
/* Counter/Timer Group B		*/
#define DAQ_CTB0		0x18
#define DAQ_CTB1		0x19
#define DAQ_CTB2		0x1a
#define DAQ_CTBMODE		0x1b
/* Digital I/O group			*/
#define DAQ_DIGA		0x10
#define DAQ_DIGB		0x11
#define DAQ_DIGC		0x12
#define DAQ_DIGMODE		0x13

/* Interval Counter Group		*/
#define DAQ_INTVLDATA		0x1e
#define DAQ_INTVLSTROBE		0x1f

/* How many IO ports do we use ? */
#define LABPCIOSIZE		0x20

/* Mask for the bit fields in each register */
/* DAQ Status Register - DAQ_STAT */
#define	DAVAIL			0x01	/* ADC conversn data available 	*/
#define	ADC_OVERRUN		0x02	/* ADC overrun error		*/
#define	ADC_OVERFLOW		0x04	/* ADC FIFO overflow error	*/
#define	CNTINT			0x08	/* State of ctr A2 interrupt	*/
#define	DMATC			0x10	/* Set if DMA transfer complete	*/
#define	GATA0			0x20	/* Status of Gate on counter A0	*/
#define	EXTGATA0		0x40	/* Status of external trigger	*/
#define	UNUSED1			0x80

/* Command Register 1 - DAQ_CMD1 */
#define	MA0			0x01	/* Select from 8 ADC input chans*/
#define	MA1			0x02
#define	MA2			0x04
#define	TWOSCMP			0x08	/* Sign extend ADC to 16 bits?	*/
#define	GAIN0			0x10	/* ADC gain setting		*/
#define	GAIN1			0x20
#define	GAIN2			0x40
#define	SCANEN			0x80	/* Enable multi-chan scanning	*/

/* Command Register 2 - DAQ_CMD2 */
#define	PRETRIG			0x01	/* Set pretriggering		*/
#define	HWTRIG			0x02	/* Set external triggering	*/
#define	SWTRIG			0x04	/* Software triggering		*/
#define	TBSEL			0x08	/* Select timebase for ADC	*/
#define	TWOSDAC0		0x10	/* Binary coding scheme for DAC	*/
#define	TWOSDAC1		0x20
#define	LDAC0			0x40	/* Set DAC waveform generation	*/
#define	LDAC1			0x80

/* Command Register 3 - DAQ_CMD3 */
#define	DMAEN			0x01	/* Enable DMA requests		*/
#define	DIOINTEN		0x02	/* Enable interrupt on DIO	*/
#define	TCINTEN			0x04	/* Enable int. DMA terminate	*/
#define	CNTINTEN		0x08	/* Enable int. on counters	*/
#define	ERRINTEN		0x10	/* Enable int. on ADC error	*/
#define	FIFOINTEN		0x20	/* Enable int. on ADC data ready*/
#define	UNUSED2			0x40
#define UNUSED3			0x80

/* Command Register 4 - DAQ_CMD4 */
#define	INTSCAN			0x01	/* Enable interval scanning	*/
#define	EOIRCV			0x02	/* Set clock for interval scan	*/
#define	ECKDRV			0x04	/* Dirn. of EXTCONV on IO line	*/
#define	SE_D			0x08	/* ADC single_ended/differential*/
#define	UNUSED4			0x10
#define	UNUSED5			0x20
#define	UNUSED6			0x40
#define UNUSED7			0x80

/* Struct for accessing the LabPC registers via the I/O ports */
typedef struct DaqPort_t {
	int	value;			/* Saved value of this register	*/
	int	address;		/* Port address of this register*/
} DaqPort_t;

typedef struct CmdRegs_t {
	DaqPort_t	stat;		/* LabPC status port		*/
	DaqPort_t	cmd1;		/* LabPC command port		*/
	DaqPort_t	cmd2;		/* LabPC command port		*/
	DaqPort_t	cmd3;		/* LabPC command port		*/
	DaqPort_t	cmd4;		/* LabPC command port		*/
} CmdRegs_t;

typedef struct LabPC_t {
	int		loaded;		/* Is the device loaded		*/
	int		major;		/* Major # for this card	*/
	int		present;	/* A LabPC card is present	*/
	int		open;		/* The device is open		*/
	int		stat;		/* Device status		*/
	int		base;		/* Base address of card		*/
	int		irq;		/* Interrupt number of card	*/
	int		dma;		/* DMA channel for DMA t/fers	*/
	CmdRegs_t	regs;		/* The board command registers	*/
	struct Counter_t *counter;	/* The 8253 counter driver	*/
	struct Dac_t	*dac;		/* The Digital to Analogue Conv	*/
	struct ADC_t	*adc;		/* The Analogue to Digital Conv */
	struct Dio_t	*dio;		/* Digital I/O ports		*/
} LabPC_t;

extern LabPC_t LabPC;

/*
 * Macros to access LabPC control ports
 * We need to store the settings of each control register. This enables
 * us to set/clear specific bits, without affecting other control bits.
 */

/* Set the bits in the register at port address according to mask */
#define setbits( port, mask )						\
{ 									\
	  unsigned long flags; save_flags( flags ); cli(); 		\
	  outb_p( (port)->value |= (mask), (port)->address ); 		\
	  SLOW_DOWN_IO ; restore_flags( flags ) ; 			\
}

/* Clear the bits in the register at port address according to mask */
#define clearbits( port, mask )						\
{ 									\
	  unsigned long flags; save_flags( flags ); cli(); 		\
	  outb_p( (port)->value &= ~(mask), (port)->address ); 		\
	  SLOW_DOWN_IO; restore_flags( flags ); 			\
}

/* Set the bits in the register at port address according to set/clear mask */
#define writebits( port, setmask, clearmask )				\
{ 									\
	  unsigned long flags; save_flags( flags ); cli(); 		\
	  outb_p( (port)->value = ((port)->value | (setmask)) & ~(clearmask), \
		  (port)->address ); 					\
	  SLOW_DOWN_IO; restore_flags( flags ); 			\
}

/* Write a word to the designated port */
#define writeport( port, word )						\
{ 									\
	  unsigned long flags; save_flags( flags ); cli(); 		\
	  outb_p( (port)->value = word, (port)->address ) ; 		\
	  SLOW_DOWN_IO; restore_flags( flags ) ; 			\
}

/* Read a word from the designated port */
#define readport( port )	\
	((port)->value = inb_p( (port)->address ))

#define GETCOUNTER( dev ) 						\
	(((MINOR(dev)) >= DAQ_COUNTERA0 &&  				\
	  (MINOR(dev)) <= DAQ_COUNTERB2)				\
	 ? &(getboard(MAJOR(dev))->counter[(MINOR(dev))-DAQ_COUNTERA0])	\
	 : NULL )

#define FINDCOUNTER( board, minor )					\
	(((minor) >= DAQ_COUNTERA0 &&  					\
	  (minor) <= DAQ_COUNTERB2)					\
	 ? &((board)->counter[(minor)-DAQ_COUNTERA0])			\
	 : NULL )

#define GETDAC( dev )							\
	(((MINOR(dev)) >= DAQ_DAC0 &&					\
	  (MINOR(dev)) < DAQ_DAC0 + NUM_DACS)				\
	 ? &(getboard(MAJOR(dev))->dac[(MINOR(dev))-DAQ_DAC0]) 		\
	 : NULL )

#define GETADC( dev )							\
	(getboard(MAJOR(dev))->adc)

#define GETDIO( dev ) 	\
	((MINOR(dev) >= DAQ_DPORTA && MINOR(dev) <= DAQ_DPORTC) ?	\
	 	(getboard(MAJOR(dev))->dio) : NULL )

LabPC_t *getboard( int major );

/* Global Variables */
extern struct file_operations counter_fops;
extern struct file_operations dac_fops;
extern struct file_operations adc_fops;
extern struct file_operations dio_fops;

#define printregs( board ) \
	printk( "Registers: cmd1=0x%x cmd2=0x%x cmd3=0x%x cmd4=0x%x stat=0x%x\n", (board)->regs.cmd1.value, (board)->regs.cmd2.value, (board)->regs.cmd3.value, (board)->regs.cmd4.value, readport( &((board)->regs.stat) ) )

#ifdef DBG_LABPC
#define	dprintk( a )		printk a
#define dprintregs( board )	printregs( board )
#else  /* DBG_LABPC */
#define dprintregs( board )
#define dprintk( a )
#endif /* DBG_LABPC */

void labpc_close( struct inode *inode, struct file *file );
int labpc_ioctl_check( unsigned int cmd, unsigned long arg );

#endif /* __KERNEL__ */

/* The magic number for LABPC ioctl codes
   - see /usr/src/linux/MAGIC & /usr/src/linux/include/asm/iotcl.h */
#ifndef LABPC_MAGIC
#define LABPC_MAGIC		'l'
#endif

/* Ioctl values - I'm using the ioctl numbering scheme described in
 * /usr/src/linux/MAGIC and /usr/src/linux/include/asm/ioctl.h */
#define LABPC_STATUS		_IO ( LABPC_MAGIC, 1 )

/* The Counters... */
#define NUM_COUNTERS		(DAQ_COUNTERB2 - DAQ_COUNTERA0 + 1)

/* Ioctls values */
#define COUNTER_SET_MODE	_IO ( LABPC_MAGIC, 11 )
#define COUNTER_GET_MODE	_IOR( LABPC_MAGIC, 12, unsigned long )

/* Counter configuration modes */
#define MODE_TERMINAL_COUNT	0	/* See Data sheet for 8253	*/
#define MODE_ONE_SHOT		1
#define MODE_RATE_GENERATOR	2
#define MODE_SQUARE_WAVE	3
#define MODE_SOFT_STROBE	4
#define MODE_HARD_STROBE	5
#define NUMBER_OF_MODES		6

/* The DACs... */
#define	NUM_DACS		2

#define DAC_SET_MODE		_IO ( LABPC_MAGIC, 21 )
#define DAC_GET_MODE		_IOR( LABPC_MAGIC, 22, unsigned long )
#define DAC_SET_CLOCK		_IO ( LABPC_MAGIC, 23 )
#define DAC_GET_CLOCK		_IOR( LABPC_MAGIC, 24, unsigned long )
#define DAC_SET_NUMTRACES	_IO ( LABPC_MAGIC, 25 )
#define DAC_GET_NUMTRACES	_IOR( LABPC_MAGIC, 26, unsigned long )
#define DAC_START		_IO ( LABPC_MAGIC, 27 )
#define DAC_STOP		_IO ( LABPC_MAGIC, 28 )
#define DAC_SET_POLARITY	_IO ( LABPC_MAGIC, 29 )
#define DAC_GET_POLARITY	_IOR( LABPC_MAGIC, 30, unsigned long )
#define DAC_SET_TIMEBASE	_IO ( LABPC_MAGIC, 31 )
#define DAC_GET_TIMEBASE	_IOR( LABPC_MAGIC, 32, unsigned long )

/* Mode selection definitions */
#define DAC_WRITE_MODE		1
#define DAC_TRACE_MODE		2
#define DAC_TRACE_NOWAIT_MODE	3
#define DAC_WAVEFORM_MODE	4

/* DAC encoding mode (bipolar or unipolar output) */
#define DAC_UNIPOLAR		0
#define DAC_BIPOLAR		1

/* ADC ioctl codes */
/* Commands */
#define ADC_GET_STATUS		_IOR( LABPC_MAGIC, 40, unsigned long )
#define ADC_INIT		_IO ( LABPC_MAGIC, 41 )
#define ADC_CLEAR		_IO ( LABPC_MAGIC, 42 )
#define ADC_TRIGGER		_IO ( LABPC_MAGIC, 43 )
#define ADC_ACQUIRE		_IO ( LABPC_MAGIC, 44 )
#define ADC_STOP		_IO ( LABPC_MAGIC, 45 )
/* Options */
#define ADC_SET_SCAN		_IO ( LABPC_MAGIC, 50 )
#define ADC_GET_SCAN		_IOR( LABPC_MAGIC, 51, unsigned long )
#define ADC_SET_GAIN		_IO ( LABPC_MAGIC, 52 )
#define ADC_GET_GAIN		_IOR( LABPC_MAGIC, 53, unsigned long )
#define ADC_SET_POLARITY	_IO ( LABPC_MAGIC, 54 )
#define ADC_GET_POLARITY	_IOR( LABPC_MAGIC, 55, unsigned long )
#define ADC_SET_CHANNEL		_IO ( LABPC_MAGIC, 56 )
#define ADC_GET_CHANNEL		_IOR( LABPC_MAGIC, 57, unsigned long )
#define ADC_SET_TIMEBASE	_IO ( LABPC_MAGIC, 58 )
#define ADC_GET_TIMEBASE	_IOR( LABPC_MAGIC, 59, unsigned long )
#define ADC_SET_TRIGGER_MODE	_IO ( LABPC_MAGIC, 60 )
#define ADC_GET_TRIGGER_MODE	_IOR( LABPC_MAGIC, 61, unsigned long )
#define ADC_SET_DMA		_IO ( LABPC_MAGIC, 62 )
#define ADC_GET_DMA		_IOR( LABPC_MAGIC, 63, unsigned long )
#define ADC_SET_CONNECTION	_IO ( LABPC_MAGIC, 64 )
#define ADC_GET_CONNECTION	_IOR( LABPC_MAGIC, 65, unsigned long )
#define ADC_SET_ECKDRV		_IO ( LABPC_MAGIC, 66 )
#define ADC_GET_ECKDRV		_IOR( LABPC_MAGIC, 67, unsigned long )
#define ADC_SET_INTSCAN		_IO ( LABPC_MAGIC, 68 )
#define ADC_GET_INTSCAN		_IOR( LABPC_MAGIC, 69, unsigned long )
#define ADC_SET_INTCOUNT	_IO ( LABPC_MAGIC, 70 )
#define ADC_GET_INTCOUNT	_IOR( LABPC_MAGIC, 71, unsigned long )
#define ADC_SET_INTCLOCK	_IO ( LABPC_MAGIC, 72 )
#define ADC_GET_INTCLOCK	_IOR( LABPC_MAGIC, 73, unsigned long )
#define ADC_SET_CLOCK		_IO ( LABPC_MAGIC, 74 )
#define ADC_GET_CLOCK		_IOR( LABPC_MAGIC, 75, unsigned long )
#define ADC_SET_AUTOACQUIRE	_IO ( LABPC_MAGIC, 76 )
#define ADC_GET_AUTOACQUIRE	_IOR( LABPC_MAGIC, 77, unsigned long )
#define ADC_SET_BUFSIZE		_IO ( LABPC_MAGIC, 78 )
#define ADC_GET_BUFSIZE		_IOR( LABPC_MAGIC, 79, unsigned long )
#define ADC_SET_NBUF		_IO ( LABPC_MAGIC, 80 )
#define ADC_GET_NBUF		_IOR( LABPC_MAGIC, 81, unsigned long )

/* ADC_GET_STATUS bit masks */
#define ADC_STATUS_ACTIVE		1
#define ADC_STATUS_INACTIVE		2
#define ADC_STATUS_STOPPED		4
#define ADC_STATUS_STARTING		8
#define ADC_STATUS_OVERRUN		16
#define ADC_STATUS_OVERFLOW		32
#define ADC_STATUS_BUFFER_OVERFLOW	64
#define ADC_STATUS_IO_ERROR		128
#define ADC_STATUS_ACQUISITION_COMPLETE	256
#define ADC_STATUS_ERROR		\
	(ADC_STATUS_OVERRUN | ADC_STATUS_OVERFLOW | \
	 ADC_STATUS_BUFFER_OVERFLOW | ADC_STATUS_IO_ERROR)

/* Allowed values for ADC_SET_GAIN */
#define ADC_GAIN_1		0
#define ADC_GAIN_1p25		1
#define ADC_GAIN_2		2
#define ADC_GAIN_5		3
#define ADC_GAIN_10		4
#define ADC_GAIN_20		5
#define ADC_GAIN_50		6
#define ADC_GAIN_100		7
#define ADC_GAIN_MAX		ADC_GAIN_100

/* Allowed values for ADC_SET_POLARITY */
#define ADC_UNIPOLAR		0
#define ADC_BIPOLAR		1

/* Allowed values for ADC_SET_CONNECTION */
#define ADC_SINGLEENDED		0
#define ADC_DIFFERENTIAL	1

/* Number of available channels = 8 (channel 0-7) */
#define ADC_NUM_CHANNELS	8

/* Allowed values for ADC_SET_TRIGGER_MODE */
#define ADC_SOFTTRIGGER		0
#define ADC_HARDTRIGGER		1
#define ADC_PRETRIGGER		2
#define ADC_AUTOTRIGGER		3

/* Ioctl values for 8255 Digital IO modes */
#define	DIO_SETBIT		_IO ( LABPC_MAGIC, 90 )	
#define DIO_RESETBIT		_IO ( LABPC_MAGIC, 91 )
#define DIO_SETMODE		_IO ( LABPC_MAGIC, 92 )

#endif /* _LINUX_LABPC_H */
