# Initialise the array of ADC config variables
proc initadcdata {dev} {
    set arr device$dev
    global ${arr}
    global config${arr}

    # Create the data array of configuration variables
    $dev configure -timebaseunits milliseconds
    foreach i [$dev configure] {
	set ${arr}([string trimleft [lindex $i 0] "-"]) [lindex $i 1]
    }
    setadctimebase ${arr} {} w

    trace variable ${arr} w "configadc $dev"
    trace variable ${arr}(clock) w "setadctimebase"
    trace variable ${arr}(timebase) w "setadctimebase"
    trace variable ${arr}(intclock) w "setadctimebase"
    trace variable ${arr}(intcount) w "setadctimebase"

    $dev configure -nonblock on
    $dev configure -autoacquire off
}

# Trace the device config variables and perform required commands
proc configadc {device arr name op} {
    global $arr

    if {$op == "w"} {
	$device configure -$name [set ${arr}($name)]
    }
}

# Update the textvariable for displaying timebase & samplerate info
proc setadctimebase {arr name op} {
    global $arr
    global config$arr

    set timebase [set ${arr}(timebase)]
    set clock [set ${arr}(clock)]
    set period [expr $clock * $timebase / 2.0]

    # Choose the most appropriate units for the timebase display
    if {$period < 1} {
	set T_units {microseconds}
	set T_value $period
	set F_units {kHz}
	set F_value 0
    } elseif {$period < 2000} {
	set T_units {microseconds}
	set T_value $period
	set F_units {kHz}
	set F_value [expr 1000.0 / $period]
    } elseif {$period < 2000000} {
	set T_units {milliseconds}
	set T_value [expr $period / 1000.0]
	set F_units {Hz}
	set F_value [expr 1000000.0 / $period]
    } else {
	set T_units {seconds}
	set T_value [expr $period / 1000000.0]
	set F_units {Hz}
	set F_value [expr 1000000.0 / $period]
    }

    # Set the display message string
    set config${arr}(fstring) "Frequency: $F_value $F_units\nPeriod: $T_value $T_units"

    set intclock [set ${arr}(intclock)]
    set period [expr $intclock * $timebase / 2.0]

    # Choose the most appropriate units for the timebase display
    if {$period < 1} {
	set T_units {microseconds}
	set T_value $period
	set F_units {kHz}
	set F_value 0
    } elseif {$period < 2000} {
	set T_units {microseconds}
	set T_value $period
	set F_units {kHz}
	set F_value [expr 1000.0 / $period]
    } elseif {$period < 2000000} {
	set T_units {milliseconds}
	set T_value [expr $period / 1000.0]
	set F_units {Hz}
	set F_value [expr 1000000.0 / $period]
    } else {
	set T_units {seconds}
	set T_value [expr $period / 1000000.0]
	set F_units {Hz}
	set F_value [expr 1000000.0 / $period]
    }
    
    # Set the display message string
    set config${arr}(intfstring) "Frequency: $F_value $F_units\nPeriod: $T_value $T_units"
}

proc AdcAcquire {adc graph b} {
    set arr device${adc}
    global ${arr}
    global config${arr}

    $adc handler "AdcHandler $adc $graph $b"

    $b configure -state disabled
    set config${arr}(numpoints) [set config${arr}(nsamples)]
    set config${arr}(ydata) {}
    $adc acquire [set config${arr}(nsamples)]
    set config${arr}(xdata) [$adc readtimebase [set config${arr}(nsamples)]]
}

proc AdcHandler {adc graph b} {
    set arr device${adc}
    global ${arr}
    global config${arr}

    set numpoints [set config${arr}(numpoints)]
    set script "$adc read $numpoints"
    puts $script
    if {catch "$script" data} {
	puts "Error: $data"
    } else {
	puts "NoError: $data"
    }
    if {[set nchan [llength $data]]} {
	set odata [set config${arr}(ydata)]
	set config${arr}(numpoints) \
		[expr $numpoints - [llength [lindex $data 0]]]

	set ndata {}
	for {set i 0} {$i < $nchan} {incr i} {
	    lappend ndata [concat [lindex $odata $i] [lindex $data $i]]
	}
	set config${arr}(ydata) $ndata
	if {[set config${arr}(numpoints)] <= 0} {
	    after idle adcdisplaydata $adc $graph
	}
    }
    if {[$adc status] == "inactive"} {
	$b configure -state normal
    }
}

# Actually perform an acquisition and display the results
proc adcdisplaydata {adc graph} {
    set arr device${adc}
    global ${arr}
    global config${arr}

    # Get info we need for display from the device data array
    set gain [set ${arr}(gain)]
    set clock [set ${arr}(clock)]
    set timebase [set ${arr}(timebase)]
    set scanning [set ${arr}(scan)]
    set numpoints [set config${arr}(nsamples)]
    set lastchan [set ${arr}(channel)]
    set timebaseunits [set ${arr}(timebaseunits)]
    set firstchan $lastchan
    if {$scanning} {
	set firstchan 0
    }
    set nchan [expr $lastchan - $firstchan + 1]
    set linewidth [option get $graph linewidth Linewidth]

    set xdata [set config${arr}(xdata)]
    set ydata [set config${arr}(ydata)]

    # Delete any old displayed graphs
    foreach i [$graph names] {
	$graph delete $i
    }

    $graph configure -xlabel "Time ($timebaseunits)"
    if {[llength $ydata] > 0} {
	for {set i $firstchan} {$i <= $lastchan} {incr i} {
	    if {[llength [lindex $ydata [expr $i - $firstchan]]] > 0} {
		$graph insert ${adc}_channel$i \
			-xdata [lindex $xdata [expr $i - $firstchan]] \
			-ydata [lindex $ydata [expr $i - $firstchan]] \
			-label "Channel $i" \
			-linewidth $linewidth \
			-color [option get $graph linecolor$i Linecolor$i]
	    }
	}
    }
}

# Make the menubar widget for the ADC
proc makeadcmenubar {w arr} {
    global ${arr}

    # Build the ADC menubar
    frame $w -borderwidth 2 -relief raised

    # The File menu
    set m $w.filemenu
    menubutton $m \
	    -menu "$m.m" \
	    -text {File}
    menu $m.m
    $m.m add command \
	    -command "destroy [winfo toplevel $w]" \
	    -label {Quit}
 
    # The ADC menu
    set m $w.adcmenu
    menubutton $m \
	    -text {ADC} \
	    -menu "$m.m"
    menu $m.m
    $m.m add checkbutton \
	    -label {DMA mode} \
	    -onvalue "on" \
	    -offvalue "off" \
	    -variable ${arr}(dma)
    $m.m add command \
	    -label {Buffer size} \
	    -command "dialogvalue $w.dialog \"ADC Buffer Size\" {Set the internal buffer size\n(Number of samples)} ${arr} buffersize"

    # The Help Menu
    set m $w.helpmenu
    menubutton $m \
	    -menu "$m.m" \
	    -text {Help}
    menu $m.m
    $m.m add command \
	    -label {LabPC} \
	    -command "popuphelp labpc"
    $m.m add command \
	    -label {ADC} \
	    -command "popuphelp ADC"

    # Pack them all together
    pack append $w \
	    $w.filemenu {left frame center} \
	    $w.adcmenu {left frame center} \
	    $w.helpmenu {right frame center}
}

proc makeadcdisplay {w dev arr} {
    global ${arr}
    global config${arr}

    # Build the analogue display widget
    frame $w

    # Build the graph widget (a frame wrapper so we can set the class)
    frame $w.graph \
	    -class Graph
    xygraph $w.graph.display
    set bg [option get $w.graph.display background Background]
    if {"$bg" == ""} {
	$w.graph.display configure -background [$w cget -background]
    } else {
	$w.graph.display configure -background "$bg"
    }
    pack append $w.graph \
	    $w.graph.display {left frame center fillx filly padx 10 pady 10 expand}

    # Build the control panel
    set p $w.panel
    frame $p \
	    -relief raised \
	    -borderwidth 2
    # ADC Acquire button
    button $p.acquire \
	    -text "Acquire" \
	    -command "AdcAcquire $dev $w.graph.display $p.acquire"
    bind $p.acquire <Any-ButtonPress-3> "popuphelp \"adc acquire\""

    # Number of Readings
    set config${arr}(nsamples) 100
    labelentry $p.nsamples "Number of Readings" config${arr}(nsamples)
    set config${arr}(nsamples) [option get $p.nsamples nsamples Nsamples]

    # Scan Enable checkbutton
    checkbutton $p.scan \
	    -text "Scan [set ${arr}(scan)]" \
	    -onvalue "on" \
	    -offvalue "off" \
	    -variable ${arr}(scan) \
	    -command "$p.scan configure -text \"Scan \[set ${arr}(scan)\]\""
    bind $p.scan <Any-ButtonPress-3> "popuphelp ${arr}(scan)"

    # Interval Scan Enable checkbutton
    checkbutton $p.intscan \
	    -text "Interval Mode [set ${arr}(intscan)]" \
	    -onvalue "on" \
	    -offvalue "off" \
	    -variable ${arr}(intscan) \
	    -command "$p.intscan configure -text \"Interval Mode \[set ${arr}(intscan)\]\""
    bind $p.intscan <Any-ButtonPress-3> "popuphelp ${arr}(intscan)"

    # Channel number
    combo $p.channel ${arr} ${arr}(channel) "Channel" {0 1 2 3 4 5 6 7}

    # Analogue gain
    combo $p.gain ${arr} ${arr}(gain) "Gain" {1 1.25 2 5 10 20 50 100}

    # Polarity
    combo $p.polarity ${arr} ${arr}(polarity) "Polarity" {bipolar unipolar}

    # Trigger Mode
    combo $p.trigger ${arr} ${arr}(trigger) "Trigger" {auto soft hard pre}

    # Connection Type
    combo $p.connection ${arr} ${arr}(connection) "Connection" {singleended differential}

    # Pack the buttons into the control panel
    pack append $p \
	    $p.acquire		{top pady 5} \
	    $p.nsamples		{top pady 5} \
	    $p.intscan		{top fillx pady 5} \
	    $p.scan		{top fillx pady 5} \
	    $p.channel		{top fillx pady 5} \
	    $p.gain		{top fillx pady 5} \
	    $p.polarity		{top fillx pady 5} \
	    $p.trigger		{top fillx pady 5} \
	    $p.connection	{top fillx pady 5}

    # Pack the display widget
    pack append $w \
	    $w.graph {left frame center fillx filly expand} \
	    $w.panel {right frame center filly}
}

# Create a widget for displaying the timebase information
proc makeadctimebase {w arr} {
    global ${arr}
    global config${arr}

    # Build the control panel
    frame $w \
	    -relief raised \
	    -borderwidth 2

    # Build the control panel
    frame $w.left \
	    -relief flat \
	    -borderwidth 0
    labelentry $w.left.clock "ADC Clock\n(counts)" "${arr}(clock)"
    labelentry $w.left.intclock "Interval Clock\n(counts)" "${arr}(intclock)"
    pack append $w.left \
	    $w.left.clock {top frame e} \
	    $w.left.intclock {top frame e}
    
    frame $w.middle \
	    -relief flat \
	    -borderwidth 0
    labelentry $w.middle.timebase "Clock Timebase\n(microseconds)" \
	    "${arr}(timebase)"
    labelentry $w.middle.intcount "Interval Count\n(Number of Samples)" \
	    "${arr}(intcount)"
    pack append $w.middle \
	    $w.middle.timebase {top frame e} \
	    $w.middle.intcount {top frame e}
    
    frame $w.right \
	    -relief flat \
	    -borderwidth 0
    label $w.right.summary \
	    -justify left \
	    -textvariable "config${arr}(fstring)"
    label $w.right.intsummary \
	    -justify left \
	    -textvariable "config${arr}(intfstring)"
    pack append $w.right \
	    $w.right.summary {top frame w} \
	    $w.right.intsummary {top frame w}

    pack append $w \
	    $w.left {left padx 5} \
	    $w.middle {left padx 5} \
	    $w.right {left padx 5}
}

# procedure to show window
proc adcwidget {w adc} {
    global device$adc
    set arr device$adc

    initadcdata $adc

    catch "destroy $w"

    # The encasing frame for the entire widget
    frame $w \
	    -class ADC

    # Build the widget menubar
    makeadcmenubar $w.menubar $arr

    # Build the analogue display widget
    makeadcdisplay $w.display $adc $arr

    # Make the timbase information display panel
    makeadctimebase $w.timebase $arr

    pack append $w \
	    $w.menubar {top frame center fillx} \
	    $w.display {top fillx filly expand} \
	    $w.timebase {bottom frame center fillx}
}

